*************************************************************************
* Program 13 BEEPER.ASM  To give continuous sound with beeper according *
* to switches on port A and flash lights with hardware delay loop       *
* Hardware JP2 Optional, JP4 Optional, JP5 ground, JP6 Vcc              *
* JP1 set to PC0.                                                       *
* For louder sound short one or more of Port C Pins to PC0              *
* JP2 lights the LEDs at the same frequency                             *
* JP4 lights the whole of the & segment display                         *
* Duration is read in from switches on port B                           *
*************************************************************************
* define addresses
PORTA   EQU  0  ;Port A - Switches
PORTB   EQU  1  ;Port B - Lights
PORTC   EQU  2  ;Port C - Beeper (bit zero) 
DDRA    EQU  4  ;Port A Direction register
DDRB    EQU  5  ;Port B Direction register
DDRC    EQU  6  ;Port C Direction register
COUNT   EQU  50         ;Count 50 times = 0.1 ms
TSR     EQU  $13        ;Timer Status Register
OCFLAG  EQU  $06        ;Output Compare Flag - bit 6 in TSR
OCHR    EQU  $16        ;Output Compare Register High byte
OCLR    EQU  $17        ;Output Compare Register Low byte
CHR     EQU  $18        ;Counter High byte
CLR     EQU  $19        ;Counter Low byte

        ORG  $50    ;RAM area
TICKS   RMB  1      ;save one location for duration value

        ORG  $250       ;start of program area
START:  lda  #0         ;set all bits of port A as input
        sta  DDRA       ;by setting DDRA to 0
        lda  #$FF       ;Port B set to output
        sta  DDRB       ;set all bits in DDRB to 1
        sta  DDRC       ;C o/p

LOOP:   lda  #0         ;load zero to Acc
        sta  PORTB      ;and output to lights
        sta  PORTC      ;and beeper
        jsr  DELAY      ;goto delay subroutine
        lda  #$FF       ;load all ones to Acc
        sta  PORTB      ;and output to lights
        sta  PORTC      ;and Beeper
        jsr  DELAY      ;goto delay subroutine
        bra  LOOP       ;branch to label LOOP

DELAY:  lda   PORTA             ;read switches - delay time
        sta   TICKS             ;save in RAM
        lda   CHR               ;read counter high byte
        sta   OCHR              ;and write to compare register
        lda   CLR               ;repeat for counter
        sta   OCLR              ;low byte
LOOP1:  lda   OCLR              ;read last low byte of compare register
        add   #{COUNT%256}      ;add low byte of count
        tax                     ;save it temporarily in X
        lda   OCHR              ;read last high byte of compare register
        adc   #{COUNT/256}      ;add high byte of count plus any carry
        sta   OCHR              ;store Acc in high byte of compare register
        stx   OCLR              ;store X in low byte of compare register
HOLD:   brclr  OCFLAG,TSR,HOLD  ;hold until counter = register
        dec   TICKS             ;decrement ticks
        bne   LOOP1             ;repeat until TICKS = 0
        rts                     ;return to main program

        END                     ;end of program

