*************************************************************************
* Program DIMMER.ASM  -To read the Switches (A) for duration and        *
* Rotate the LEDs on (B)                                                *
* Adjust brightness by converting voltage on potentiometer VR1 to       *
* digital value and outputting value as analog signal at pulse length   *
* modulation register to drive LEDs                                     *
* Hardware JP1 PC0, JP2 Out, JP3 IN, JP4 Out                            *
*************************************************************************
*Define Port Addresses
PORTA EQU 0          ;Port A (switches) 
PORTB EQU 1          ;Port B (LEDs) 
DDRA EQU 4           ;Data Direction Register for PortA
DDRB EQU 5           ;Data Direction Register for PortB
ADDATA   EQU  8      ;A/D data register
ADSTAT   EQU  9      ;A/D status register
PLMA     EQU  $A     ;Pulse length modulation register

        ORG $50         ;RAM area
TICKS RMB 1             ;save one location for duration value
TEMPA RMB 1             ;temp storage for Acc

        ORG $250        ;start of program area

START: lda #0           ;set all bits of port A as input
       sta DDRA         ;by setting DDRA to 0
       lda #$FF         ;Port B set to output
       sta DDRB         ;by setting DDRB to all 1's
       lda #$FE         ;first value to accumulator
       sec              ;set carry bit
LOOP:  sta PORTB        ;Output
       rola             ;rotate bit to left
       bcs OK           ;branch round when carry set
       lda #$FE         ;else reset to beginning
       sec              ;set carry bit


OK:    sta  TEMPA               ;temp storage for Acc
       jsr DELAY                ;Jump to subroutine delay
       lda    #$20              ;set ADC for channel 0, system clock, start
       sta    ADSTAT            ;Start acquisition
WAIT:  brclr  7,ADSTAT,WAIT     ;wait for completion
       lda    ADDATA            ;get value
       sta    PLMA              ;and store in D/A output
       lda TEMPA                ;reload temp value of Acc
       bra LOOP                 ;branch to label LOOP and continue

DELAY: lda PORTA ;Software delay. Read switches for delay time. 
       sta TICKS ;save in RAM
LOOP1: lda #20T  ;middle loop counter
LOOP2: ldx #166T ;inner loop counter
LOOP3: decx      ;decrement inner counter
       bne LOOP3 ;end of inner loop
       deca      ;decrement middle loop counter
       bne LOOP2 ;end of middle loop
       dec TICKS ;decrement outer loop counter
       bne LOOP1 ;end of outer loop

       rts       ;return from subroutine

   END           ;end prog


