*************************************************************************
* Program 4 HARDFLSH.ASM To flash LEDS (PortB) using hardware delay loop*
* with duration read from switches (PortA)                              *
* Hardware JP2 in, JP4 Optional, JP5 ground, JP6 Vcc                    *
*************************************************************************
*define addresses
PORTA EQU 0     ;Port A (switches) 
PORTB EQU 1     ;Port B (LEDs) 
DDRA EQU 4              ;Data Direction Register for PortA
DDRB EQU 5              ;Data Direction Register for PortB
COUNT   EQU 5000T       ;Timer clock cycles (base ten) 
TSR     EQU $13 ;Timer status register
OCFLAG  EQU 6   ;Bit 6 of status register is compare flag
OCHR    EQU $16 ;Compare register 1 high
OCLR    EQU $17 ;Compare register 1 low
CHR     EQU $18 ;Counter high register
CLR     EQU $19 ;Counter low register

    ORG $50
TICKS RMB 1
    ORG $250            ;Storage for value from switches
START: lda #0           ;set all bits of port A as input
       sta DDRA         ;by setting DDRA to 0
       lda #$FF         ;Port B set to output
       sta DDRB         ;by setting DDRB to all 1's
LOOP:  lda #0           ;As Ports must sink current to turn on
       sta PORTB        ;Zeros set LEDs on
       jsr DELAY        ;Jump to subroutine delay
       lda #$FF         ;Ones set LEDs off
       sta PORTB        ; 
       jsr DELAY        ;Delay again
       bra LOOP         ;Repeat
*
DELAY: lda PORTA                ;Store value
       sta TICKS                ;from switches
       lda CHR                  ;read current value of counter H
       sta OCHR                 ;store in compare register H
       lda CLR                  ;same for counter low byte
       sta OCLR                 ;store in compare register L
LOOP1: lda OCLR                 ;read last low byte of compare register
       add #{COUNT%256T}        ;add low byte of count
       tax                      ;save temporarily in X register
       lda OCHR                 ;read last high byte of compare register
       adc #{COUNT/256T}        ;add in high byte of count plus any carry
       sta OCHR                 ;store Acc in high byte of compare register
       stx OCLR                 ;store X in low byte of compare register
HOLD:  brclr OCFLAG,TSR,HOLD    ;hold until counter = register
       dec TICKS                        ;decrement TICKS
       bne LOOP1                        ;repeat until TICKS = 0
       rts                              ;return to main program
       END                              ;end of program


