***********************************************************************
* Program 10 KEY.ASM To read 4x4 keyboard on PortA and                *
* Display on 7 segment display on PortB                               *
* Hard ware :JP5 to ground, JP6 to Vcc, All dip switches S2 open      *
* JP4 in JP3 out                                                      *
* KeyPad connected as follows                                         *
*    Pin 1 col 1 to PA3 of port A: Pin 5  Row 1 to PA7 of port A      *
*        2     2    PA2                6      2    PA6                *
*        3     3    PA1                7      3    PA5                *
*        4     4    PA0                8      4    PA4                *
***********************************************************************
*                                                                     *
* This puts Vcc on the first Row then checks each column in turn to   *
* see if Vcc is present. Then puts Vcc on second Row and checks each  *
* column in turn, and so on incrementing the X reg until a key is     *
* detected. The corresponding code is then O/P from the table. If no  *
* input is detected, the sequence repeats.                            *
*                                                                     *
***********************************************************************
PORTA   EQU  0
PORTB   EQU  1
PORTC   EQU  2
DDRA    EQU  4
DDRB    EQU  5
DDRC    EQU  6 ; Ports and data direction registers
*
           org $50
STROBE    RMB  1     ; To select row
COL       RMB  1     ; To select col
KEY       RMB  1     ; To save for future use
TEMPX     RMB  1
*
           org $250
START:  lda    #$F0  ; Set Most Significant Nibble for O/P
        sta    DDRA  ; and LSN for I/P
        lda    #$FF
        sta    DDRB  ; O/P
        sta    PORTB
NXTDIG: clrx
        lda    #4
        sta    TEMPX      ; Initialise TEMPX
        lda    #%10000000 ; First ROW
        sta    STROBE
        sta    PORTA
        ldx    #0         ; Set to first index
RESTCOL:lda    #%00001000 ; First COL
        sta    COL        ; 
NXTCOL: lda    PORTA
        and    #$0F       ; Just keep I/P bits
        cmp    COL        ; Is present col selected
        beq    GETCODE    ; Goto code table
        lsr    COL        ; Check next col
        incx              ; 
        cpx    TEMPX      ; Have all col been checked
        beq    INCRX      ; 
        bra    NXTCOL     ; No, do next column along
INCRX:  lsr    STROBE     ; Yes, Now do next row
        lda    STROBE     ; 
        sta    PORTA      ; O/P next column
        txa               ; Current index to acc
        add    #4         ; add 4
        sta    TEMPX
        cpx    #$10       ; If been round all retry
        bne    RESTCOL    ; Reset to first col
        bra    NXTDIG

*
GETCODE:lda   LIST,X      ; Get correct code and
        sta   KEY         ; Store for future use
        sta   PORTB       ; O/P
        bra   NXTDIG      ; Start again

*
* Codes for 8 segment display with decimal point, 0 turns on segment
*

LIST:   fcb   %11111001,%10100100,%10110000,%10001000 ;8 seg for'1','2','3','A'
        fcb   %10011001,%10010010,%10000010,%10000011 ;  '4','5','6','b'
        fcb   %11111000,%10000000,%10011000,%11000110 ;'7','8','9','C'
        fcb   %11111111,%11000000,%01111111,%10100001 ;' ','0','.','D'


