**************************************************************************
* Program 12 KEY.LCD To drive LCD Display and display Key pressed        *
* Hardware jumpers for devices on port B must be removed:                *
* J4 (7-seg) and J2 (LEDS) to limit sink current and ensure good V low   *
* PB0 to PB7 Data, PC5 Enable, PC6 Read/write, PC7 RS control line       *
* to distinguish between control instruction and data                    *
* Dip switches should be all open                                        *
**************************************************************************
PORTA   EQU  0          ;Define Port addresses
PORTB   EQU  1
PORTC   EQU  2
DDRA    EQU  4          ;and Data direction registers
DDRB    EQU  5
DDRC    EQU  6
*
           org $50
TEMPX     RMB  1         ;Temporary storage for Acc
TEMPA     RMB  1         ;& X in delay loops
STROBE    RMB  1         ;To select row
COL       RMB  1
SUM       RMB  1         ;To keep track of index for KeyPad data
*
           org $250

START:  lda    #$F0      ;Set Most Significant Nibble for O/P
        sta    DDRA      ;and LSN for I/P
        lda    #$FF      ;set ports
        sta    DDRC      ;B and C to output
        sta    DDRB
        jsr    DINIT     ;initialise LCD Display
RPT:    jsr    READKEY   ;read keyboard - stay here until key pressed
        jsr    DISPCDE   ;send value to LCD Display
        jsr    HOLD      ;wait until key released
        bra    RPT       ;Repeat loop forever
*
READKEY:clrx
        lda    #%10000000 ;First ROW
        sta    STROBE

NXTROW: lda    STROBE
        sta    PORTA
        lsr    STROBE     ;Shift one bit to right for next time

        lda    PORTA      ;read KeyPad
        and    #$0F       ;Just keep I/P bits (col) 

        beq    NOKEY      ;no key pressed, try next row
        jsr    LNGDEL     ;Now long delay
        lda    PORTA      ; 
        and    #$0F
        bne    KEYFND


NOKEY:  txa               ;Add 4 to index (X) 
        add    #4         ;to get to next row
        tax
        cpx    #$10       ;check to see if been to all rows
        bne    NXTROW     ;if not, move to next row
        bra    READKEY    ;else start again
KEYFND: incx
        sta    sum
        add    sum
        bhcc   KEYFND     ;branch if zero in bit 4
        decx              ; 
        rts               ;Exit with key data in X
*
HOLD:   lda    PORTA      ;read KeyPad
        and    #$0F       ;cols only
        bne    HOLD       ;loop if a key still pressed
        rts

*
DISPCDE:lda   LIST,X      ;Get correct code and
        jsr   WDAT        ;O/P
        rts               ; 
*
DINIT:  clrx               ;clear index
MORE:   lda   INITDAT,X    ;read data
        jsr   WCNTL        ;and output
        incx               ;repeat
        cpx   #6           ;six times
        bne   MORE
        rts
*
WDAT:   sta    PORTB       ;data output on PORTB
        bset   5,PORTC     ;latch data with
        bclr   5,PORTC     ;falling edge of Enable
        jsr    SHTDEL
        rts
*
WCNTL:  sta    PORTB       ;data output on PORTB
        bclr   7,PORTC     ;set RS to CONTROL
        bset   5,PORTC     ;latch control with
        bclr   5,PORTC     ;falling edge of Enable
        bset   7,PORTC     ;set RS to DATA
        cmp    #2          ;long delay if 'clear display'
        bls    LNG         ;or 'home cursor; i.e. 2 ms
        jsr    SHTDEL      ;else 40 microsec delay
        bra    DONE
LNG:    bsr    LNGDEL      ;get long delay
DONE:   rts

* Short Delay subroutine
SHTDEL: stx    TEMPX       ;X used
        ldx    #20         ;in counting loop
LOOP:   decx               ;for 40 microsec delay
        bne    LOOP
        ldx    TEMPX       ;restore X
        rts

* Long delay subroutine
LNGDEL: sta    TEMPA       ;calls
        lda    #30         ;short delay 30 times
AGN:    jsr    SHTDEL
        deca
        bne    AGN
        lda    TEMPA
        rts
*
INITDAT: fcb  $38,$38,$38,$0C,6,1     ;data to set up LCD Display -
*                                     ;8 bit, 2 line, no cursor
LIST:    fcb  '123A456B789C 0.D'      ;Data in correct order for KeyPad


