**************************************************************************
* Program 11 LCD.ASM To drive LCD Display and display predefined message *
* Hardware jumpers for devices on port B must be removed:                *
* J4 (7-seg) and J2 (LEDS) to limit sink current and ensure good V low   *
* PB0 to PB7 Data, PC5 Enable, PC6 Read/write, PC7 RS control line       *
* to distinguish between control instruction and data                    *
**************************************************************************
PORTA   EQU  0          ;Define port addresses
PORTB   EQU  1
PORTC   EQU  2
DDRA    EQU  4          ;and data direction registers
DDRB    EQU  5
DDRC    EQU  6
*
           org $50
TEMPX     RMB  1         ;temporary storage for Acc & X
TEMPA     RMB  1
*
           org $250
START:  lda    #$FF      ;set ports
        sta    DDRC      ;B and C to output
        sta    DDRB
        jsr    DINIT     ;initialise LCD Display
        jsr    MSOUT     ;send message
FIN:    bra FIN          ;and hold
*
*
MSOUT:  clrx              ;set index to zero
MESS:   lda MESSGE,x      ;read message char by char
        cmp   #0          ;until zero found
        beq  NOMORE       ;then exit
        jsr WDAT          ;output char
        incx              ;and get next one
        bra MESS
NOMORE: rts
*
DINIT:  clrx               ;clear index
MORE:   lda   INITDAT,X    ;read data
        jsr   WCNTL        ;and output
        incx               ;repeat
        cpx   #6           ;six times
        bne   MORE
        rts
*
WDAT:   sta    PORTB       ;data output on PORTB
        bset   5,PORTC     ;latch data with
        bclr   5,PORTC     ;falling edge of E
        jsr    SHTDEL
        rts
*
WCNTL:  sta    PORTB       ;data output on PORTB
        bclr   7,PORTC     ;set RS to CONTROL
        bset   5,PORTC     ;latch control with
        bclr   5,PORTC     ;falling edge of E
        bset   7,PORTC     ;set RS to DATA
        cmp    #2          ;long delay if 'clear display'
        bls    LNG         ;or 'home cursor; i.e. 2 ms
        jsr    SHTDEL      ;else 40 microsec delay
        bra    DONE
LNG:    bsr    LNGDEL      ;get long delay
DONE:   rts
*
SHTDEL: stx    TEMPX       ;X used
        ldx    #20         ;in counting loop
LOOP:   decx               ;for 40 microsec delay
        bne    LOOP
        ldx    TEMPX       ;restore X
        rts
*
LNGDEL: sta    TEMPA       ;calls
        lda    #30         ;short delay 30 times
AGN:    jsr    SHTDEL
        deca
        bne    AGN
        lda    TEMPA
        rts
*
INITDAT: fcb  $38,$38,$38,$0C,6,1     ;data to set up LCD Display -
*                                     ;8 bit, 2 line, no cursor
MESSGE:  fcb  'HELLO WORLD AGAIN!',0  ;message string ending with 0


