*************************************************************************
* Program 9 SINE.ASM To generate a sine wave at PLMB centred at 2.5 V 	*
* with period approximately 55 ms					*
* Hardware O/P J1, if JP4 in will illuminate all segments		*
* R1 (47K) and C1 (0.1 microF) for smooth display			*
*************************************************************************
*
$BASE  10T			;Set default base to ten
*
PORTB   EQU  1		;Define addresses
DDRB    EQU  5
LO      EQU  40		;Values for
HI      EQU  2		;hardware delay
TSR     EQU  $13		;Timer status register
OCFLAG  EQU  6		;Bit 6 for compare flag
OCHR    EQU  $16		;Output Compare Register High byte
OCLR    EQU  $17		;Output Compare Register Low byte
CHR     EQU  $18		;Counter High Register
CLR     EQU  $19		;Counter Low Register
DATAMAX EQU  50		;50 values of data
PLMB    EQU  $0B		;Pulse Length Modulation register B
*
        org $50
INDEXB   RMB  1		;Storage for index
*
         org $250
START:  lda    #$FF	;Set Port B for O/P
        sta    DDRB
        lda    CHR	;Read counter high byte
        sta    OCHR	;store compare high
        lda    CLR	;same for 
        sta    OCLR	;low byte
LOOP:   jsr    SINEB	
        jsr    DELAY
        bra    LOOP
*
DELAY:  lda    OCLR			;
        add    #LO			;add in low byte delay
        tax					;save temporarily in X
        lda    OCHR			;load compare high byte
        adc    #HI			;add with any carry
        sta    OCHR			;Now have counter 
        stx    OCLR			;plus delay in compare registers
WAIT:   brclr  OCFLAG,TSR,WAIT	;Wait until flag is set
        rts
*
SINEB:  ldx    INDEXB			;Get last stored value of index
        lda    DATA,X			;for data from table
        sta    PLMB			;O/P at PLMB
        incx				;
        cpx    #DATAMAX			;reached end of data?
        blo    SAVEB			;No branch round
        clrx				;Yes reset to zero
SAVEB:  stx    INDEXB			;Store X value
        rts
*
DATA:  FCB  128,144,160,175,190,203,215,226,235,243
       FCB  249,253,255,255,253,249,243,235,226,215
       FCB  203,190,175,160,144,128,112,96,81,66
       FCB  53,41,30,21,13,7,3,1,1,3
       FCB  7,13,21,30,41,53,66,81,96,112
*
_

